@@ -2,6 +2,7 @@ import xldeploy
 import pkg_resources
 import platform
 import traceback
+import getpass
 
 from os import environ, path
 from sys import argv
@@ -9,8 +10,10 @@ from xld.xld_args_validator import XldArgsValidator
 from xld.help import help_message
 from xld.help_deploy import deploy_usage
 from xld.help_apply import apply_usage
-from xld.utility import find_uploads, read_files
-from xldeploy.errors import APIError, XLDeployAuthenticationError
+from xld.help_command import help_usage
+from xld.help_generate import generate_usage
+from xld.utility import decode_escapes, find_reads, find_uploads, read_files
+from xldeploy.errors import APIError, XLDeployConnectionError, XLDeployConnectionTimeout, XLDeployAuthenticationError
 
 
 class XldCommands:
@@ -37,10 +40,12 @@ class XldCommands:
 
         try:
             deploy_file = open(file_path, 'rb').read()
-            matched_file = find_uploads(deploy_file)
+            matched_file = find_uploads(deploy_file) + find_reads(deploy_file)
             files_to_upload = read_files(matched_file, path.abspath(path.join(file_path, "..")))
-            # print(files_to_upload)
-            return self.__client.deployfile.apply(file_path, files_to_upload)
+            response_data = self.__client.deployfile.apply(file_path, files_to_upload)
+            if not response_data or len(response_data) == 0 or response_data == '[]':
+                return "No changes required to update XL Deploy"
+            return ''.join(list(map(lambda changeSet: changeSet.replace("\\", ""), response_data))).replace(",", "\n")
         except Exception as e:
             return self.__handle_error(e)
 
@@ -65,8 +70,26 @@ class XldCommands:
         except Exception as e:
             return self.__handle_error(e)
 
-    def help(self):
-        return help_message
+    def encrypt(self):
+        plainText = getpass.getpass(prompt='What is plain text to encrypt?')
+        try:
+            return self.__client.deployfile.encrypt(plainText)
+        except Exception as e:
+            return self.__handle_error(e)
+
+    def help(self, command=None):
+        if command is not None and command == 'apply':
+            return apply_usage.format(xld_url='http://localhost:4516', xld_username='john', xld_password='secret01')
+        elif command is not None and command == 'deploy':
+            return deploy_usage.format(xld_url='http://localhost:4516', xld_username='john', xld_password='secret01')
+        elif command is not None and command == 'generate':
+            return generate_usage.format(xld_url='http://localhost:4516', xld_username='john', xld_password='secret01')
+        elif command is not None and command == 'help':
+            return help_usage
+        elif command is not None and command == 'version':
+            return self.version()
+        else:
+            return help_message
 
     def version(self):
         packagedir = path.abspath(path.join(__file__, "../"))
@@ -81,6 +104,11 @@ class XldCommands:
         message = ""
         if isinstance(error, XLDeployAuthenticationError):
             message = "The username or password you have entered is incorrect."
+        elif isinstance(error, XLDeployConnectionError):
+            message = "Could not connect to XL Deploy server, please check if server is running at: {xld_url}." \
+                .format(xld_url=self.__url)
+        elif isinstance(error, XLDeployConnectionTimeout):
+            message = "Timeout connecting to {xld_url}".format(xld_url=self.__url)
         elif isinstance(error, APIError):
             if error.is_client_error():
                 message = "Client Error:\n"
@@ -91,4 +119,4 @@ class XldCommands:
             message = error.__str__()
         if self.__debug:
             traceback.print_stack()
-        return message
+        return decode_escapes(message)
